@@ -24,10 +24,13 @@
 #include <sys/time.h>
 #include "cc-timezone-map.h"
 #include "set-timezone.h"
+#include "dt-lockbutton.h"
+
 #include <gsettings-desktop-schemas/gdesktop-enums.h>
 #include <string.h>
 #include <stdlib.h>
 #include <libintl.h>
+#include <polkit/polkit.h>
 
 #define GETTEXT_PACKAGE_TIMEZONES GETTEXT_PACKAGE "-timezones"
 
@@ -518,7 +521,7 @@ get_regions (TzLocation             *loc,
   /* Load the translation for it */
   zone = g_strdup (dgettext (GETTEXT_PACKAGE_TIMEZONES, loc->zone));
   g_strdelimit (zone, "_", ' ');
-  split_translated = g_strsplit_set (zone, TRANSLATION_SPLIT, 2);
+  split_translated = g_regex_split_simple ("[\\x{2044}\\x{2215}\\x{29f8}\\x{ff0f}/]", zone, 0, 0);
   g_free (zone);
 
   if (!g_hash_table_lookup_extended (data->table, split[0], NULL, NULL))
@@ -721,6 +724,22 @@ queue_clock_update (CcDateTimePanel *self)
   self->priv->update_id = g_timeout_add (timeouttime, (GSourceFunc)update_time_timer, self);
 }
 
+static void
+on_permission_changed (GPermission *permission,
+                       GParamSpec  *pspec,
+                       gpointer     data)
+{
+  CcDateTimePanelPrivate *priv = CC_DATE_TIME_PANEL (data)->priv;
+  gboolean allowed;
+  GtkWidget *vbox;
+
+  allowed = g_permission_get_allowed (permission);
+
+  vbox = (GtkWidget*) gtk_builder_get_object (priv->builder, "vbox");
+
+  gtk_widget_set_sensitive (vbox, allowed);
+}
+
 static void
 cc_date_time_panel_init (CcDateTimePanel *self)
 {
@@ -737,6 +756,8 @@ cc_date_time_panel_init (CcDateTimePanel *self)
   guint i, num_days;
   gboolean using_ntp;
   int ret;
+  GtkWidget *lockbutton;
+  GPermission *permission;
 
   priv = self->priv = DATE_TIME_PANEL_PRIVATE (self);
 
@@ -839,11 +860,26 @@ cc_date_time_panel_init (CcDateTimePanel *self)
                                           NULL);
 
   queue_clock_update (self);
+
+  /* add the lock button */
+  permission = polkit_permission_new_sync ("org.gnome.settingsdaemon.datetimemechanism.configure", NULL, NULL, NULL);
+
+  /* DtLockButton takes ownership of the permission */
+  lockbutton = dt_lock_button_new (permission);
+  gtk_widget_set_margin_top (lockbutton, 12);
+  gtk_widget_show (lockbutton);
+  gtk_box_pack_end ((GtkBox *) gtk_builder_get_object (priv->builder, "hbox"),
+                    lockbutton, FALSE, FALSE, 0);
+  g_signal_connect (permission, "notify",
+                    G_CALLBACK (on_permission_changed), self);
+  on_permission_changed (permission, NULL, self);
 }
 
 void
 cc_date_time_panel_register (GIOModule *module)
 {
+  bind_textdomain_codeset (GETTEXT_PACKAGE_TIMEZONES, "UTF-8");
+
   cc_date_time_panel_register_type (G_TYPE_MODULE (module));
   g_io_extension_point_implement (CC_SHELL_PANEL_EXTENSION_POINT,
                                   CC_TYPE_DATE_TIME_PANEL,
