@@ -111,14 +111,16 @@ static void
 xkb_options_select (gchar * optionname)
 {
 	gboolean already_selected = FALSE;
-	gchar **options_list = xkb_options_get_selected_list ();
-	if (options_list != NULL) {
-		gchar **option = options_list;
-		while (*option != NULL)
-			if (!strcmp (*option, optionname)) {
-				already_selected = TRUE;
-				break;
-			}
+	gchar **options_list;
+	guint i;
+
+	options_list = xkb_options_get_selected_list ();
+	for (i = 0; options_list != NULL && options_list[i] != NULL; i++) {
+		gchar *option = options_list[i];
+		if (!strcmp (option, optionname)) {
+			already_selected = TRUE;
+			break;
+		}
 	}
 
 	if (!already_selected) {
@@ -415,11 +417,8 @@ static void
 chooser_response_cb (GtkDialog * dialog, gint response, gpointer data)
 {
 	switch (response) {
-	case GTK_RESPONSE_HELP:
-		/* capplet_help (GTK_WINDOW (dialog),
-		   "prefs-keyboard-layoutoptions"); */
-		break;
-	case GTK_RESPONSE_CLOSE:{
+	case GTK_RESPONSE_DELETE_EVENT:
+	case GTK_RESPONSE_CLOSE: {
 			/* just cleanup */
 			GSList *expanders_list =
 			    g_object_get_data (G_OBJECT (dialog),
@@ -448,13 +447,12 @@ xkb_options_popup_dialog (GtkBuilder * dialog)
 
 	chooser = CWID ("xkb_options_dialog");
 	gtk_window_set_transient_for (GTK_WINDOW (chooser),
-				      GTK_WINDOW (WID ("region_dialog")));
+				      GTK_WINDOW (gtk_widget_get_toplevel (WID ("region_notebook"))));
 	xkb_options_load_options (chooser_dialog);
 
 	g_signal_connect (chooser, "response",
 			  G_CALLBACK (chooser_response_cb), dialog);
-
-	gtk_dialog_run (GTK_DIALOG (chooser));
+	gtk_widget_show (chooser);
 }
 
 /* Update selected option counters for a group-bound expander */
