@@ -86,12 +86,11 @@ static void     gvc_level_bar_class_init (GvcLevelBarClass *klass);
 static void     gvc_level_bar_init       (GvcLevelBar      *level_bar);
 static void     gvc_level_bar_finalize   (GObject            *object);
 
-G_DEFINE_TYPE (GvcLevelBar, gvc_level_bar, GTK_TYPE_HBOX)
+G_DEFINE_TYPE (GvcLevelBar, gvc_level_bar, GTK_TYPE_WIDGET)
 
 #define check_rectangle(rectangle1, rectangle2)                          \
         {                                                                \
-                if (rectangle1.x != rectangle2.x) return TRUE;           \
-                if (rectangle1.y != rectangle2.y) return TRUE;           \
+                /* .x and .y are always 0 */                             \
                 if (rectangle1.width  != rectangle2.width)  return TRUE; \
                 if (rectangle1.height != rectangle2.height) return TRUE; \
         }
@@ -438,12 +437,7 @@ static void
 gvc_level_bar_size_request (GtkWidget      *widget,
                             GtkRequisition *requisition)
 {
-        GvcLevelBar *bar;
-
-        g_return_if_fail (GVC_IS_LEVEL_BAR (widget));
-        g_return_if_fail (requisition != NULL);
-
-        bar = GVC_LEVEL_BAR (widget);
+        GvcLevelBar *bar = GVC_LEVEL_BAR (widget);
 
         switch (bar->priv->orientation) {
         case GTK_ORIENTATION_VERTICAL:
@@ -460,6 +454,40 @@ gvc_level_bar_size_request (GtkWidget      *widget,
         }
 }
 
+static void
+gvc_level_bar_get_preferred_width (GtkWidget *widget,
+                                   gint      *minimum,
+                                   gint      *natural)
+{
+        GtkRequisition requisition;
+
+        gvc_level_bar_size_request (widget, &requisition);
+
+        if (minimum != NULL) {
+                *minimum = requisition.width;
+        }
+        if (natural != NULL) {
+                *natural = requisition.width;
+        }
+}
+
+static void
+gvc_level_bar_get_preferred_height (GtkWidget *widget,
+                                    gint      *minimum,
+                                    gint      *natural)
+{
+        GtkRequisition requisition;
+
+        gvc_level_bar_size_request (widget, &requisition);
+
+        if (minimum != NULL) {
+                *minimum = requisition.height;
+        }
+        if (natural != NULL) {
+                *natural = requisition.height;
+        }
+}
+
 static void
 gvc_level_bar_size_allocate (GtkWidget     *widget,
                              GtkAllocation *allocation)
@@ -550,17 +578,11 @@ gvc_level_bar_draw (GtkWidget *widget,
                     cairo_t   *cr)
 {
         GvcLevelBar     *bar;
-        GtkAllocation   allocation;
 
         g_return_val_if_fail (GVC_IS_LEVEL_BAR (widget), FALSE);
 
         bar = GVC_LEVEL_BAR (widget);
 
-        gtk_widget_get_allocation (widget, &allocation);
-        cairo_translate (cr,
-                         allocation.x,
-                         allocation.y);
-
         if (bar->priv->orientation == GTK_ORIENTATION_VERTICAL) {
                 int i;
                 int by;
@@ -648,7 +670,8 @@ gvc_level_bar_class_init (GvcLevelBarClass *klass)
         object_class->get_property = gvc_level_bar_get_property;
 
         widget_class->draw = gvc_level_bar_draw;
-        widget_class->size_request = gvc_level_bar_size_request;
+        widget_class->get_preferred_width = gvc_level_bar_get_preferred_width;
+        widget_class->get_preferred_height = gvc_level_bar_get_preferred_height;
         widget_class->size_allocate = gvc_level_bar_size_allocate;
 
         g_object_class_install_property (object_class,
