@@ -22,6 +22,7 @@
  *
  */
 
+#include <math.h>
 #include "cc-ua-panel.h"
 
 #include <gconf/gconf-client.h>
@@ -323,12 +324,10 @@ static gdouble
 dpi_from_pixels_and_mm (gint pixels,
                         gint mm)
 {
-  gdouble dpi;
-
   if (mm >= 1)
     return pixels / (mm / 25.4);
   else
-    return dpi = 0;
+    return 0;
 }
 
 static gdouble
@@ -366,6 +365,8 @@ get_dpi_from_x_server ()
   return dpi;
 }
 
+static void dpi_combo_box_changed (GtkComboBox *box, CcUaPanel *panel);
+
 static void
 dpi_notify_cb (GSettings   *settings,
                const gchar *key,
@@ -378,6 +379,8 @@ dpi_notify_cb (GSettings   *settings,
   gboolean valid;
   gdouble conf_value;
   gdouble x_dpi;
+  GtkTreeIter best;
+  gdouble distance;
 
   if (!g_str_equal (key, "dpi"))
     return;
@@ -390,30 +393,31 @@ dpi_notify_cb (GSettings   *settings,
   /* get current value from screen */
   x_dpi = get_dpi_from_x_server ();
 
-  /* see if the calculated value matches in the combobox model */
+  /* find the closest match in the combobox model */
+  distance = 1e6;
   valid = gtk_tree_model_get_iter_first (model, &iter);
   while (valid)
     {
       gfloat factor;
+      gdouble d;
 
       gtk_tree_model_get (model, &iter,
                           DPI_MODEL_FACTOR_COLUMN, &factor,
                           -1);
 
-      if (conf_value == (float) (factor * x_dpi))
+      d = fabs (conf_value - factor * x_dpi);
+      if (d < distance)
         {
-          gtk_combo_box_set_active_iter (GTK_COMBO_BOX (combo), &iter);
-          break;
+          best = iter;
+          distance = d;
         }
 
       valid = gtk_tree_model_iter_next (model, &iter);
     }
 
-  /* if a matching value was not found in the combobox, set to "normal" */
-  if (!valid)
-    {
-      gtk_combo_box_set_active (GTK_COMBO_BOX (combo), 0);
-    }
+  g_signal_handlers_block_by_func (combo, dpi_combo_box_changed, panel);
+  gtk_combo_box_set_active_iter (GTK_COMBO_BOX (combo), &best);
+  g_signal_handlers_unblock_by_func (combo, dpi_combo_box_changed, panel);
 }
 
 static void
@@ -573,6 +577,16 @@ visual_bell_type_toggle_cb (GtkWidget *button,
     gconf_client_set_string (panel->priv->client, key, "fullscreen", NULL);
 }
 
+static void
+hearing_sound_preferences_clicked (GtkButton *button,
+                                   CcUaPanel *panel)
+{
+  CcShell *shell;
+
+  shell = cc_panel_get_shell (CC_PANEL (panel));
+  cc_shell_set_active_panel_from_id (shell, "sound", NULL);
+}
+
 static void
 cc_ua_panel_init_hearing (CcUaPanel *self)
 {
@@ -609,6 +623,9 @@ cc_ua_panel_init_hearing (CcUaPanel *self)
   g_signal_connect (WID (priv->builder, "hearing_test_flash_button"),
                     "clicked", G_CALLBACK (gdk_beep), NULL);
 
+  g_signal_connect (WID (priv->builder, "hearing_sound_preferences_button"),
+                    "clicked",
+                    G_CALLBACK (hearing_sound_preferences_clicked), self);
 }
 
 /* typing/keyboard section */
@@ -741,7 +758,7 @@ cc_ua_panel_init (CcUaPanel *self)
                        "pointing_sizegroup", "pointing_sizegroup2",
                        "pointing_scale_sizegroup", "sizegroup1",
                        "hearing_sizegroup",
-                       "NULL" };
+                       NULL };
 
   priv = self->priv = UA_PANEL_PRIVATE (self);
 
